import subprocess

from .types import IsoEnv, IsoEnvArgs, PyProjectToml, Requirements


def iso_run(
    args: IsoEnvArgs, cmd_list: list[str], **process_args
) -> subprocess.CompletedProcess:
    """Runs the command using the isolated environment."""
    from .run import run

    return run(args, cmd_list, **process_args)


def iso_open_process(
    args: IsoEnvArgs, cmd_list: list[str], **process_args
) -> subprocess.Popen:
    """Runs the command using the isolated environment."""
    from .run import open_proc

    return open_proc(args, cmd_list, **process_args)


__all__ = ["IsoEnv", "IsoEnvArgs", "Requirements", "PyProjectToml", "IsoEnv"]



import os
import subprocess
import warnings
from pathlib import Path

from iso_env.api import install, installed, purge
from iso_env.types import IsoEnvArgs
from iso_env.util import get_verbose_from_env, to_full_cmd_list


def run(
    args: IsoEnvArgs,
    cmd_list: list[str],
    verbose: bool | None = None,
    **process_args,
) -> subprocess.CompletedProcess:
    """Runs the command using the isolated environment."""
    verbose = verbose if verbose is not None else get_verbose_from_env()
    if not installed(args, verbose=verbose):
        purge(args.venv_path)
        install(args, verbose=verbose)
    env = dict(os.environ)
    if "env" in process_args:
        env = process_args.pop("env")
    if "VIRTUAL_ENV" in env:
        del env["VIRTUAL_ENV"]
    full_cmd_list = to_full_cmd_list(args, cmd_list, verbose=verbose, **process_args)
    check = process_args.pop("check", True)
    try:
        cp = subprocess.run(full_cmd_list, env=env, check=check, **process_args)
        return cp
    except subprocess.CalledProcessError as exc:
        stderr = exc.stderr
        stdout = exc.stdout
        error_msg = f"Failed to run: {exc.cmd} {exc.returncode}\nstdout: {stdout}\nstderr: {stderr}"
        warnings.warn(error_msg)
        raise


def open_proc(
    args: IsoEnvArgs,
    cmd_list: list[str],
    verbose: bool | None = None,
    **process_args,
) -> subprocess.Popen:
    """Runs the command using the isolated environment."""
    verbose = verbose if verbose is not None else get_verbose_from_env()
    if not installed(args, verbose=verbose):
        purge(args.venv_path)
        install(args, verbose=verbose)
    full_cmd_list = to_full_cmd_list(args, cmd_list, verbose=verbose, **process_args)
    shell = process_args.pop("shell", False)
    if verbose:
        full_path = Path(".").resolve()
        full_cmd_str = subprocess.list2cmdline(full_cmd_list)
        print(f"Running in {full_path}: {full_cmd_str}")

    env = dict(os.environ)
    if "env" in process_args:
        env = process_args.pop("env")
    if "VIRTUAL_ENV" in env:
        del env["VIRTUAL_ENV"]

    proc = subprocess.Popen(  # type: ignore
        full_cmd_list,
        shell=shell,
        env=env,
        **process_args,
    )
    return proc



import subprocess
from dataclasses import dataclass
from pathlib import Path


@dataclass
class Requirements:
    content: str
    python_version: str | None = None

    def __post_init__(self):
        self.content = self.content.strip()

    def __repr__(self):
        return self.content


@dataclass
class PyProjectToml:
    content: str

    def __post_init__(self):
        self.content = self.content.strip()

    def __repr__(self):
        return self.content


@dataclass
class IsoEnvArgs:
    venv_path: Path
    build_info: Requirements | PyProjectToml


@dataclass
class EnvironmentPartition:
    env: dict[str, str]
    process_args: dict[str, str]


class IsoEnv:
    def __init__(self, args: IsoEnvArgs) -> None:
        self.args = args

    def run(self, cmd_list: list[str], **process_args) -> subprocess.CompletedProcess:
        from iso_env.run import run

        return run(self.args, cmd_list, **process_args)

    def open_proc(self, cmd_list: list[str], **process_args) -> subprocess.Popen:
        from iso_env.run import open_proc

        return open_proc(self.args, cmd_list, **process_args)



"""
Unit test file.
"""

import os
import unittest
from pathlib import Path

from iso_env import IsoEnv, IsoEnvArgs, Requirements

os.environ["ISO_ENV_VERBOSE"] = "1"

REQUIREMENTS_TXT = """
"""


class PurgeOnChangeTester(unittest.TestCase):
    """Main tester class."""

    def test_iso_env_with_cwd(self) -> None:
        """Test command line interface (CLI)."""
        venv_path = Path(".env_purge").resolve()
        args = IsoEnvArgs(
            venv_path=venv_path,
            build_info=Requirements(REQUIREMENTS_TXT),
        )
        iso = IsoEnv(args)
        cp = iso.run(["pwd"], capture_output=True, text=True, check=True)
        self.assertEqual(cp.returncode, 0)
        # now add a new line to the requirements file
        args.build_info = Requirements(REQUIREMENTS_TXT + "pytest")
        iso = IsoEnv(args)
        cp = iso.run(["pwd"], capture_output=True, text=True, check=True)
        self.assertEqual(cp.returncode, 0)
        gen_pyproject = venv_path / "pyproject.toml"
        self.assertTrue(gen_pyproject.exists())
        text = gen_pyproject.read_text()
        self.assertIn("pytest", text)


if __name__ == "__main__":
    unittest.main()



import os
import shutil

from iso_env.types import IsoEnvArgs


def get_verbose_from_env() -> bool:
    return os.environ.get("ISO_ENV_VERBOSE", "0") == "1"


def to_full_cmd_list(
    args: IsoEnvArgs,
    cmd_list: list[str],
    **process_args,  # needed to capture unexpected arguments
) -> list[str]:
    uv = shutil.which("uv")
    assert uv is not None, "uv not found."
    preamble = [
        uv,
        "run",
        "--project",
        str(args.venv_path),
    ]
    return preamble + cmd_list
